@@ -385,8 +385,8 @@ class DiffConfig:
     """
 
 
-    def __init__(self, init_explain=False, init_debug_config=False,
-                 init_debug_diff=False, init_debug_convert=0):
+    def __init__(self, init_explain=False, init_dump_config=False,
+                 init_dump_diff=False, init_debug_convert=0):
 
         """The constructor initialises the exclude list to empty and
         adds and converters using _add_converters().  It also stores
@@ -397,10 +397,10 @@ class DiffConfig:
         configuration changes that explain the differences being matched
         by the DiffConvert objects (if available).
 
-        init_debug_config=False -- dump the 'from' and 'to'
+        init_dump_config=False -- dump the 'from' and 'to'
         configurations (after exludes)
 
-        init_debug_diff=False -- dump the differences (remove and update
+        init_dump_diff=False -- dump the differences (remove and update
         configurations)
 
         init_debug_convert=0 -- level of debugging information for the
@@ -410,8 +410,8 @@ class DiffConfig:
 
         # store the [initial] settings for the conversion process
         self._explain = init_explain
-        self._debug_config = init_debug_config
-        self._debug_diff = init_debug_diff
+        self._dump_config = init_dump_config
+        self._dump_diff = init_dump_diff
         self._debug_convert = init_debug_convert
 
         # initialise the dictionary of excludes, which will be passed
@@ -483,7 +483,7 @@ class DiffConfig:
         return None
 
 
-    def _changes_begin(self):
+    def _diffs_begin(self):
         """This method returns a head (beginning) for a generated
         changes configuration file as an iterable of strings.
 
@@ -498,7 +498,7 @@ class DiffConfig:
         return []
 
 
-    def _changes_end(self):
+    def _diffs_end(self):
         """This method returns a tail (ending) for a generated changes
         configuration file as an iterable of strings.
 
@@ -692,19 +692,34 @@ class DiffConfig:
         copy.deepcopy()ed before passing them in, if this is
         undesirable.
 
-        The returned value is a single big string of all the
-        configuration changes that need to be made.
+        The returned value is a 2-tuple:
+
+        - the first element is a big string of all the configuration
+          changes that need to be made, sandwiched between
+          _diffs_begin() and _diffs_end(), or None, if there were no
+          differences
+
+        - the second element is a dictionary giving the tree of
+          differences (i.e. the elements where a difference was
+          encountered - either a remove or an update)
         """
 
 
         self.apply_rules(from_cfg)
 
-        if self._debug_config:
+        if self._dump_config:
             print(">> 'from' configuration (after rules, if specified):",
                   yaml.dump(dict(from_cfg), default_flow_style=False),
                   sep="\n")
 
 
+        # initialise the list of diffs (the returned configuration
+        # conversions) and the tree of differences to empty
+
+        diffs = []
+        diffs_tree = {}
+
+
         # if no 'to' config was specified, stop here (we assume we're
         # just testing, parsing and excluding items from the 'from'
         # configuration and stopping
@@ -713,12 +728,12 @@ class DiffConfig:
         # configuration and an empty configuration
 
         if to_cfg is None:
-            return None
+            return None, diffs_tree
 
 
         self.apply_rules(to_cfg)
 
-        if self._debug_config:
+        if self._dump_config:
             print(">> 'to' configuration (after rules, if specified):",
                 yaml.dump(dict(to_cfg), default_flow_style=False), sep="\n")
 
@@ -735,7 +750,7 @@ class DiffConfig:
         remove_cfg_full = deepfilter(from_cfg, remove_cfg)
 
 
-        if self._debug_diff:
+        if self._dump_diff:
             print("=> differences - remove:",
                   yaml.dump(remove_cfg, default_flow_style=False),
 
@@ -748,12 +763,6 @@ class DiffConfig:
                   sep="\n")
 
 
-        # initialise the list of changes (the returned configuration
-        # conversions)
-
-        changes = []
-
-
         # go through the list of converter objects
 
         for cvt in self._cvts:
@@ -832,28 +841,28 @@ class DiffConfig:
                     # corresponding entry from the 'from' configuration
 
                     try:
-                        change = cvt.remove(cvt_old, cvt_rem, *args)
+                        diff = cvt.remove(cvt_old, cvt_rem, *args)
 
-                    except TypeError as e:
-                        m = ("%s for converter:\n%s"
-                                 % (e, pathstr(match, cvt.wildcard_indices)))
+                    except:
+                        print("with: type(cvt)=" + repr(type(cvt)),
+                              file=sys.stderr)
 
-                        raise TypeError(m) from None
+                        raise
 
 
-                    # if some changes were returned by the action, add
+                    # if some diffs were returned by the action, add
                     # them
 
-                    if change is not None:
+                    if diff is not None:
                         # the return can be either a simple string or a
                         # list of strings - if it's a string, make it a
                         # list of one so we can do the rest the same way
 
-                        if isinstance(change, str):
-                            change = [change]
+                        if isinstance(diff, str):
+                            diff = [diff]
 
                         if self._debug_convert:
-                            print("\n".join(change))
+                            print("\n".join(diff))
 
 
                         # add a comment, explaining the match, if
@@ -862,14 +871,20 @@ class DiffConfig:
                         if self._explain:
                             comment = self._explain_comment(match)
                             if comment:
-                                changes.append(comment)
+                                diffs.append(comment)
 
 
-                        # store this change on the end of the list of
-                        # changes so far
+                        # store this diff on the end of the list of
+                        # diffs so far
+
+                        diffs.extend(diff)
+                        diffs.append("")
+
+
+                        # add this match to the differences tree
+
+                        deepsetdefault(diffs_tree, *match)
 
-                        changes.extend(change)
-                        changes.append("")
 
                     else:
                         if self._debug_convert:
@@ -913,31 +928,33 @@ class DiffConfig:
                     # call the update converter action method
 
                     try:
-                        change = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
+                        diff = cvt.update(cvt_old, cvt_upd, cvt_new, *args)
 
-                    except TypeError as e:
-                        m = ("%s for converter:\n%s"
-                                 % (e, pathstr(match, cvt.wildcard_indices)))
+                    except:
+                        print("with: type(cvt)=" + repr(type(cvt)),
+                              file=sys.stderr)
 
-                        raise TypeError(m) from None
+                        raise
 
 
-                    if change is not None:
-                        if isinstance(change, str):
-                            change = [change]
+                    if diff is not None:
+                        if isinstance(diff, str):
+                            diff = [diff]
 
                         if self._debug_convert:
-                            print("\n".join(change))
+                            print("\n".join(diff))
 
 
                         if self._explain:
                             comment = self._explain_comment(match)
                             if comment:
-                                changes.append(comment)
+                                diffs.append(comment)
 
 
-                        changes.extend(change)
-                        changes.append("")
+                        diffs.extend(diff)
+                        diffs.append("")
+
+                        deepsetdefault(diffs_tree, *match)
 
                     else:
                         if self._debug_convert:
@@ -952,11 +969,12 @@ class DiffConfig:
 
         # if nothing was generated, just return nothing
 
-        if not changes:
-            return None
+        if not diffs:
+            return None, diffs_tree
 
 
-        # return the change lines concatenated into a big, multiline
-        # string, along with the begin and end blocks
+        # return the diffs concatenated into a big, multiline string,
+        # along with the begin and end blocks
 
-        return "\n".join(self._changes_begin() + changes + self._changes_end())
+        return ("\n".join(self._diffs_begin() + diffs + self._diffs_end()),
+                diffs_tree)
