@@ -98,6 +98,23 @@ class _Cvt_Int_Shutdown(_CvtContext_Int):
 _cvts.append(_Cvt_Int_Shutdown)
 
 
+# we do VRF changes on an interface before we do any IP configuration,
+# otherwise it will be removed
+
+class _Cvt_Int_VRFFwd(_CvtContext_Int):
+    cmd = "vrf-forwarding",
+
+    def remove(self, old, rem, int_name):
+        return "interface " + int_name, " no vrf forwarding"
+
+    def update(self, old, upd, new, int_name):
+        return "interface " + int_name, " vrf forwarding " + upd
+        # TODO: need to find some way to trigger reapplication of IP
+        # information (address, HSRP, etc.)
+
+_cvts.append(_Cvt_Int_VRFFwd)
+
+
 class _Cvt_Int_ARPTime(_CvtContext_Int):
     cmd = "arp-timeout",
 
@@ -242,7 +259,7 @@ class _Cvt_Int_IPHlprAddr(_CvtContext_Int):
             l.append(" no ip helper-address " + a)
         return l
 
-    def update(self, old, upd, new, args):
+    def update(self, old, upd, new, int_name):
         l = ["interface " + int_name]
         for a in sorted(upd):
             l.append(" ip helper-address " + a)
@@ -583,20 +600,6 @@ class _Cvt_Int_SwPortTrkAlw(_CvtContext_Int):
 _cvts.append(_Cvt_Int_SwPortTrkAlw)
 
 
-class _Cvt_Int_VRFFwd(_CvtContext_Int):
-    cmd = "vrf-forwarding",
-
-    def remove(self, old, rem, int_name):
-        return "interface " + int_name, " no vrf forwarding"
-
-    def update(self, old, upd, new, int_name):
-        return "interface " + int_name, " vrf forwarding " + upd
-        # TODO: need to find some way to trigger reapplication of IP
-        # information (address, HSRP, etc.)
-
-_cvts.append(_Cvt_Int_VRFFwd)
-
-
 class _Cvt_Int_XConn(_CvtContext_Int):
     cmd = "xconnect",
 
